/*
 * gpe-applauncher
 *
 * Application launcher for GPE Phone Edition.
 * 
 * (c) kernel concepts 2007
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Device control panel.
 */
#include <stdlib.h>
#include <string.h>
#include <libintl.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <gpe/spacing.h>
#include <gpe/infoprint.h>

#include "gpe-applauncher.h"
#include "controlpanel.h"

#define _(x) gettext (x)

#define DEFAULT_THEME_DIR PREFIX "/share/themes/default/"

static void 
cpanel_do_suspend (GtkWidget *button, GtkWidget *panel)
{
  g_spawn_command_line_async ("apm -s", NULL);
}

static void 
cpanel_do_poweroff (GtkWidget *button, GtkWidget *panel)
{
  g_spawn_command_line_async ("poweroff", NULL);
}

static void 
cpanel_do_lock (GtkWidget *button, GtkWidget *panel)
{
  g_spawn_command_line_async ("gpe-lock-display", NULL);
}

void
controlpanel_open (GtkWindow *parent)
{
  GtkWidget *cpanel, *vbox;
  GtkWidget *bsuspend, *bpoweroff, *block, *bclose;
  GtkWidget *imgsuspend, *imgpoweroff, *imglock, *imgclose;
		
  cpanel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  vbox = gtk_vbox_new (TRUE, gpe_get_boxspacing());

  bsuspend = gtk_button_new_with_label (_("  Suspend"));
  bpoweroff = gtk_button_new_with_label (_("  Turn off"));
  block = gtk_button_new_with_label (_("  Lock"));
  bclose = gtk_button_new_with_label (_("  Back"));

  imgsuspend = gtk_image_new_from_file (DEFAULT_THEME_DIR "session-suspend.png");
  imgpoweroff = gtk_image_new_from_file (DEFAULT_THEME_DIR "session-halt.png");
  imglock = gtk_image_new_from_file (DEFAULT_THEME_DIR "session-lock.png");
  imgclose = gtk_image_new_from_file (DEFAULT_THEME_DIR "session-back.png");

  gtk_button_set_image (GTK_BUTTON (bsuspend), imgsuspend);
  gtk_button_set_image (GTK_BUTTON (bpoweroff), imgpoweroff);
  gtk_button_set_image (GTK_BUTTON (block), imglock);
  gtk_button_set_image (GTK_BUTTON (bclose), imgclose);
  gtk_button_set_alignment (GTK_BUTTON (bsuspend), 0.0, 0.5);
  gtk_button_set_alignment (GTK_BUTTON (bpoweroff), 0.0, 0.5);
  gtk_button_set_alignment (GTK_BUTTON (block), 0.0, 0.5);
  gtk_button_set_alignment (GTK_BUTTON (bclose), 0.0, 0.5);
		
  gtk_box_pack_start (GTK_BOX (vbox), bsuspend, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), bpoweroff, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), block, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), bclose, TRUE, TRUE, 0);

  g_signal_connect (G_OBJECT (bsuspend), "clicked", 
							G_CALLBACK (cpanel_do_suspend), cpanel);
  g_signal_connect (G_OBJECT (bpoweroff), "clicked", 
							G_CALLBACK (cpanel_do_poweroff), cpanel);
  g_signal_connect (G_OBJECT (block), "clicked", 
							G_CALLBACK (cpanel_do_lock), cpanel);
  g_signal_connect_swapped (G_OBJECT (bclose), "clicked", 
							G_CALLBACK (gtk_widget_destroy), cpanel);

	
  gtk_container_add (GTK_CONTAINER (cpanel), vbox);
	
  gtk_widget_show_all (cpanel);
  gtk_window_present (GTK_WINDOW (cpanel));
}

void
controlpanel_close (void)
{

}
