/*
 * gpe-applauncher
 *
 * Application launcher for GPE Phone Edition.
 * 
 * (c) kernel concepts 2006
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  DBus interface implementation for softkey and message interfaces.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <libintl.h>

#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>

#include "homescreen.h"
#include "dbus-object.h"
#include "applications.h"
#include "dbus-if.h"
#include "gpe-applauncher.h"

#define DBUS_SKWINDOW_IF   "org.gpe.skwindow.softkeys"
#define DBUS_SKWINDOW_PATH "/org/gpe/skwindow/softkeys"

#define _(x) gettext(x)

static DBusConnection *bus;

/* Sends a raw message to the bus without destination.
 * Clients need to react accordingly.*/
void
softkeys_send_activation (gint softkey_nr)
{
  DBusConnection *connection;
  DBusError error;
  DBusMessage *message;
  DBusMessageIter iter;

  dbus_error_init (&error);
  connection = dbus_bus_get (DBUS_BUS_SESSION, &error);

  if (connection == NULL)
    {
      g_printerr ("Failed to open connection to session bus: %s\n", error.message);
      dbus_error_free (&error);
      return;
    }
  message = dbus_message_new_signal (DBUS_SKWINDOW_PATH,
                                     DBUS_SKWINDOW_IF,
                                     "softkey_action");
  if (message == NULL)
    {
      g_printerr("Error creating message.\n");
      return;
    }
  dbus_message_set_auto_start (message, TRUE);
  dbus_message_iter_init_append (message, &iter);
  dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &softkey_nr);

  dbus_connection_send (connection, message, NULL);
  dbus_connection_flush (connection);

  dbus_message_unref (message);
}


	
void 
send_message (char * str_send)
{
#ifdef DEBUG
	g_message ("%s(): %s", __FUNCTION__, str_send);
#endif
	if (str_send == NULL)
		return;
	
	if (strlen (str_send) == 0)
		return;
	
	if (bus == NULL)
		return;
	
	DBusMessage * message = NULL;
	char *callee_name, *callee_method, *caller_service_name;
	char ** vec = g_strsplit_set (str_send, "!", -1);
	callee_name = vec[0];
	callee_method = vec[1];
	caller_service_name = vec[2];
#ifdef DEBUG
	g_message ("callee: '%s'", callee_name);
	g_message ("callee method: '%s'", callee_method);
	g_message ("caller: '%s'", caller_service_name);
#endif    
	if(!strcmp (callee_method,"run_and_hide") && !strcmp (callee_name, "Voc"))
    {
         g_strfreev (vec);
         return;
    }
	/*
	   The caller is applauncher itself, no need to send any dbus message to IAC library.
	   Instead, we send a dbus message to the callee to tell it to show itself
	   in the way defined by 'callee_method'.
	*/
	if (strcmp (caller_service_name, SERVICE_APPLAUNCHER) == 0)
	{
		if (strcmp (callee_name, VOICECALL_APP_SERVICE) == 0) //show dialer
		{
			message = dbus_message_new_signal (VOICECALL_IAC_OBJECT, VOICECALL_IAC_INTERFACE, VOICECALL_IAC_METHOD_NUM_RETURN);
			if (message != NULL)
			{				
				const gchar *arg2 = "";
				const gchar *arg3 = "";
				dbus_message_append_args (message, DBUS_TYPE_STRING, &callee_method, 
										  DBUS_TYPE_STRING, &arg2,
										  DBUS_TYPE_STRING, &arg3,
										  DBUS_TYPE_INVALID);
				dbus_message_set_destination (message, VOICECALL_IAC_SERVICE);
			}
		}		
		if (strcmp (callee_name, SMS_APP_SERVICE) == 0) //show SMS
		{
			message = dbus_message_new_method_call (SMS_IAC_SERVICE, SMS_IAC_OBJECT, SMS_IAC_INTERFACE, SMS_IAC_METHOD_INBOX);
			if (message != NULL)
			{			
				dbus_message_append_args (message, DBUS_TYPE_STRING, &callee_method,
										  DBUS_TYPE_INVALID);
			}
		}
	}
	/*
	   Send an empty dbus message to IAC library, telling the caller that
	   the callee is now up and running
	*/
	else
	{
		message = dbus_message_new_signal (IAC_OBJ_PATH, IAC_INTERFACE, IAC_METHOD);
		dbus_message_set_destination (message, caller_service_name);
	}
	
	if (message != NULL)
	{				
		dbus_connection_send (bus, message, NULL);
		dbus_message_unref (message);
	}
	
	g_strfreev (vec);
	return;
}

/* DBus message handler for the message interface we just listen on. In contrast
 * to the softkey interface we do not use an object proxy here. */
static DBusHandlerResult
msg_filter (DBusConnection *connection, DBusMessage *message, void *user_data)
{  
  /* handle presentation notifiers */
  if (dbus_message_is_signal (message, "org.gpe.applauncher.msg", "present_handler")) 
    {
      DBusError error;
      gint i;
        
      dbus_error_init (&error);
      if (dbus_message_get_args (message, &error, DBUS_TYPE_INT32, 
                                 &i, DBUS_TYPE_INVALID)) 
        {
          applauncher_present (i);
        } 
      else 
        {
          g_printerr ("Message received, but error getting message: %s\n", error.message);
          dbus_error_free (&error);
        }
      return DBUS_HANDLER_RESULT_HANDLED;
    }

  /* handle status messages */
  if (dbus_message_is_signal (message, "org.gpe.applauncher.msg", "message_display_handler")) 
    {
      DBusError error;
      char *s;
        
      dbus_error_init (&error);
      if (dbus_message_get_args (message, &error, DBUS_TYPE_STRING, 
                                 &s, DBUS_TYPE_INVALID)) 
        {
          homescreen_show_status(s);
        } 
      else 
        {
          g_printerr ("Message received, but error getting message: %s\n", error.message);
          dbus_error_free (&error);
        }
        return DBUS_HANDLER_RESULT_HANDLED;
    }
  /* handle notifications */ 
  if (dbus_message_is_signal (message, "org.gpe.applauncher.msg", "notification_display_handler")) 
    {
      DBusError error;
      char *s;
      int i;
        
      dbus_error_init (&error);
      if (dbus_message_get_args (message, &error, DBUS_TYPE_INT32, &i,
                                 DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) 
        {
          homescreen_show_notification(i, s);
        } 
      else 
        {
          g_printerr ("Message received, but error getting message: %s\n", error.message);
          dbus_error_free (&error);
        }
      return DBUS_HANDLER_RESULT_HANDLED;
    } 
    /* handle adding application description */
    if (dbus_message_is_signal (message, "org.gpe.applauncher.msg", "add_application")) 
    {
      DBusError error;
      char *s;      
        
      dbus_error_init (&error);
      if (dbus_message_get_args (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) 
        {
#ifdef DEBUG
          g_message ("applauncher: add_application %s", s);
#endif
          add_application (s);
        } 
      else 
        {
          g_printerr ("Message received, but error getting message: %s\n", error.message);
          dbus_error_free (&error);
        }
        return DBUS_HANDLER_RESULT_HANDLED;
    }
	
    /* handle removing application description */
    if (dbus_message_is_signal (message, "org.gpe.applauncher.msg", "remove_application")) 
    {
      DBusError error;
      char *s;      
        
      dbus_error_init (&error);
      if (dbus_message_get_args (message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) 
        {
#ifdef DEBUG
          g_message ("applauncher: remove_application %s", s);
#endif
          remove_application (s);
        } 
      else 
        {
          g_printerr ("Message received, but error getting message: %s\n", error.message);
          dbus_error_free (&error);
        }
        return DBUS_HANDLER_RESULT_HANDLED;
    }
  /* A signal from the bus saying we are about to be disconnected */
  if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, 
                              "Disconnected")) 
    {
      gtk_main_quit();
      return DBUS_HANDLER_RESULT_HANDLED;
    }
	
  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}

/* Initialize and set up the DBus connection. 
 * After initializing the connection the callback function for the message 
 * interface is registered. 
 * After this an object proxy for the softkey interface is created. */
gboolean
dbus_init (void)
{
  GError *error = NULL;
  DBusGConnection *connection;
  DBusGProxy *proxy;
  GObject *obj;
  guint32 request_name_ret;
  
  connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
  bus = dbus_bus_get (DBUS_BUS_SESSION, NULL);
  if (connection == NULL)
    {
      g_printerr ("Failed to get session bus connection %s", error->message);
      return FALSE;
    }
    
  /* message interface object */
  obj = g_object_new (APPLAUNCHER_IF_TYPE, NULL);
  dbus_g_connection_register_g_object (connection,
                                       "/org/gpe/applauncher/msg",
                                       obj);
  /* we do not request a namespace to support multiple listeners to these signals */
  dbus_bus_add_match (bus, "type='signal',interface='org.gpe.applauncher.msg'", NULL);
  dbus_connection_add_filter(bus, msg_filter, NULL, NULL);
  
  /* softkeys interface object */
  obj = g_object_new (APPLAUNCHER_IF_SOFTKEYS_TYPE, NULL);
  dbus_g_connection_register_g_object (connection,
                                       "/org/gpe/applauncher/softkeys",
                                       obj);
  proxy = dbus_g_proxy_new_for_name (connection,
                                              DBUS_SERVICE_DBUS,
                                              DBUS_PATH_DBUS,
                                              DBUS_INTERFACE_DBUS);
  if (!org_freedesktop_DBus_request_name (proxy, SERVICE_APPLAUNCHER,
					  0, &request_name_ret, &error))
    g_printerr ("Unable to get name %s", error->message);

  if (request_name_ret != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
    {
      g_error ("Name request returned %u", request_name_ret);
      exit (1);
    }
  
  return 0;
}
