/*
 * gpe-applauncher
 *
 * Application launcher for GPE Phone Edition.
 *
 * (c) kernel concepts 2006
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * DBus interface object.
 */
 
#include <stdlib.h>
#include <glib-object.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>

#include "dbus-object.h"
#include "DbusApplauncherGlue.h"
#include "softkeys.h"
#include "homescreen.h"
#include "applications.h"

/* Convenience macro to test and set a GError/return on failure */
#define g_set_error_val_if_fail(test, returnval, error, domain, code) G_STMT_START{ \
 if G_LIKELY (test) {} else { \
   g_set_error (error, domain, code, #test); \
   g_warning(#test " failed"); \
   return (returnval); \
 } \
}G_STMT_END

/** Object framework stuff, message object **/

/* Define Object type */
G_DEFINE_TYPE(ApplauncherIf, applauncher_if, G_TYPE_OBJECT)

GQuark
applauncher_if_error_quark (void)
{
  static GQuark quark = 0;
  if (!quark)
    quark = g_quark_from_static_string ("Some error ocurred.");
  return quark;
}

/* Class initialisation */
static void
applauncher_if_class_init (ApplauncherIfClass *aclass)
{
  dbus_g_object_type_install_info (G_OBJECT_CLASS_TYPE (aclass), &dbus_glib__object_info);
}

/* Instance initialisation */
static void
applauncher_if_init (ApplauncherIf *ai)
{
}


/** Functional stuff starts here */
gboolean
message_display_handler(ApplauncherIf *applauncher, const char* string, 
                        char **answer, GError **error)
{
  g_set_error_val_if_fail (string != NULL, FALSE, error, APPLAUNCHER_IF_ERROR, APPLAUNCHER_IF_ERROR_GENERIC);
  g_set_error_val_if_fail (answer != NULL, FALSE, error, APPLAUNCHER_IF_ERROR, APPLAUNCHER_IF_ERROR_GENERIC);
  homescreen_show_status (string);
  *answer = g_strdup ("homescreen updated, thank you");
  return TRUE;
}

gboolean
notification_display_handler(ApplauncherIf *applauncher, int i, const char* string, GError **error)
{
  g_set_error_val_if_fail (string != NULL, FALSE, error, APPLAUNCHER_IF_ERROR, APPLAUNCHER_IF_ERROR_GENERIC);
  homescreen_show_notification (i, string);
  return TRUE;
}

gboolean
present_handler(ApplauncherIf *applauncher, int i, GError **error)
{
  applauncher_present (i);
  return TRUE;
}

gboolean 
start_service_handler(ApplauncherIf *applauncher, const char* callee_name, const char* callee_method, const char* caller_service_name, int *r, GError **error)
{
  g_set_error_val_if_fail (callee_name != NULL, FALSE, error, APPLAUNCHER_IF_ERROR, APPLAUNCHER_IF_ERROR_GENERIC);
  g_set_error_val_if_fail (callee_method != NULL, FALSE, error, APPLAUNCHER_IF_ERROR, APPLAUNCHER_IF_ERROR_GENERIC);	  
  *r = applications_start_service (callee_name, callee_method, caller_service_name);	
  fprintf (stderr, "start application: %s, return: %d\n", callee_name, *r);	 
  return TRUE;
}
	
/** Object framework stuff, softkeys object **/

/* Define Object type */
G_DEFINE_TYPE(ApplauncherIfSoftkeys, applauncher_if_softkeys, G_TYPE_OBJECT)

/* Class initialisation */
static void
applauncher_if_softkeys_class_init (ApplauncherIfSoftkeysClass *aclass)
{
  dbus_g_object_type_install_info (G_OBJECT_CLASS_TYPE (aclass), &dbus_glib__object_info);
}

/* Instance initialisation */
static void
applauncher_if_softkeys_init (ApplauncherIfSoftkeys *ai)
{
}


/** Functional stuff starts here */
gboolean
update_softkey(ApplauncherIfSoftkeys *ifsoftkeys, gint nr, const char* string, GError **error)
{
  g_set_error_val_if_fail (string != NULL, FALSE, error, APPLAUNCHER_IF_ERROR, APPLAUNCHER_IF_ERROR_GENERIC);
  softkeys_update_key (nr, string);
  return TRUE;
}

gboolean 
get_amount(ApplauncherIfSoftkeys *ifsoftkeys, gint *nr, GError **error)
{
  *nr = softkeys_get_amount();
  return TRUE;
}

gboolean 
get_key(ApplauncherIfSoftkeys *ifsoftkeys, gint nr, const char **string, GError **error)
{
  *string = g_strdup(softkeys_get_key(nr));
  return TRUE;
}

gboolean 
get_menupos(ApplauncherIfSoftkeys *ifsoftkeys, gint nr, gint *menu_x, gint *menu_y, GError **error)
{
  softkeys_get_menu_pos (nr, menu_x, menu_y);
  return TRUE;
}
