/*
 * gpe-applauncher
 *
 * Application launcher for GPE Phone Edition.
 * 
 * (c) kernel concepts 2006, 2007
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Home screen display implementation.
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include <libintl.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>

#include <gpe/spacing.h>
#include <gpe/pixmaps.h>
#include <gpe/infoprint.h>
#include <gpe/launch.h>
#include <gpe/settings.h>

#include "homescreen.h"
#include "gpe-applauncher.h"
#include "dbus-if.h"
#include "applications.h"
#include "launch-app.h"

/*setting key definition*/
#define KEY_BG_IMAGE "/apps/homescreen/backgroundimage"
#define KEY_THEME_BG "/homescreen/backgroundimage"

/*file location definition*/
#define DEFAULT_THEME_BG PREFIX  "/share/themes/default/homescreen_background.png"
#define FILE_CONFIG     SYSCONFDIR "/gpe/homescreen.conf"

#define DEFAULT_STATUSTEXT  "G(PE)²"

#define _(x) gettext(x)

/* Quick launch area item type. */
typedef struct 
{
  gchar *name;
  gchar *icon;
  gchar *exec;    
}t_qlaunch_app;

/* Home screen area global data. */
static struct 
  {
    GtkWidget *box;
	GtkWidget *l_top;
	GtkWidget *l_center;
	GtkWidget *l_bottom;
    GtkWidget *q_area;
    gchar *label_colour;
    gchar *status_colour;
    gchar *status_message;
    t_qlaunch_app *q_apps;
  } homescreen;  

/* the list of popup windows for showing notifications */
static GList *popup_list;  
  
static void 
destroy_popup (gpointer data, gpointer user_data)
{
  GtkWidget *popup = GTK_WIDGET (data);
  gtk_widget_destroy (popup);
  popup_list = g_list_remove (popup_list, data);
#ifdef DEBUG
  g_message ("destroyed");
#endif
}

static gboolean 
lazy_destroy (gpointer data)
{
  gtk_widget_destroy (GTK_WIDGET (data));
  return FALSE;
}
	
static gboolean 
destroy_popup_evanascent (gpointer data)
{	
#ifdef DEBUG
  g_message ("evanascent");
#endif
  GtkWidget *popup = GTK_WIDGET (data);
  if (GTK_IS_WIDGET(popup))		
    {
      g_source_remove_by_user_data (data); /* cancel the timeout function */
      g_timeout_add (100, lazy_destroy, popup); /* to avoid "assertion `WIDGET_REALIZED_FOR_EVENT (widget, event)' failed" */
    }
  return FALSE;
}

static void 
destroy_popups (GtkWidget *widget, gint type)
{
#ifdef DEBUG
  g_message ("type is: %d", type);	
#endif
	
  g_list_foreach (popup_list, destroy_popup, NULL);
	
  if (type == NOTE_MESSAGE_SENT)
    {
      destroy_popup_evanascent (widget);
    }
}
	
static void 
activate_application (GtkWidget *widget, gint type)
{
  if (type == NOTE_MESSAGE)
    {				
      applications_start_service (SMS_APP_SERVICE, "inbox", SERVICE_APPLAUNCHER);
    }
  destroy_popups (widget, type);
}
	
static gboolean
popup_key_press_handler (GtkWidget *widget, GdkEventKey *event,
                         gpointer user_data)
{	
  gint type = GPOINTER_TO_INT (user_data);
	
  if (event->keyval == GDK_Return)
    {		
      activate_application (widget, type);
	  return TRUE;
    }
  if (event->keyval == GDK_Escape)
    {
      destroy_popups (widget, type);
      return TRUE;
    }
	
  return FALSE;
}  

static gboolean 
popup_button_press_handler (GtkWidget *widget, GdkEventButton *event,
                            gpointer user_data)
{
  gint type = GPOINTER_TO_INT (user_data);

#ifdef DEBUG
  g_message ("x,y: %f,%f", event->x, event->y);
  g_message ("root_x,root_y: %f,%f", event->x_root, event->y_root);
#endif
  if (event->x != event->x_root) /* the pointer is inside the notification window */
    activate_application (widget, type);
  else
    destroy_popups (widget, type);

  return TRUE;
}
  
/* Window size allocation handler for the notification area window. */
static gboolean
handle_size_allocate (GtkWidget *w, GtkAllocation *a)
{	
  GdkBitmap *bitmap;
  GdkGC *zero_gc, *one_gc;
  GdkColor zero, one;
  gint x0, y0, x1, y1;
  gint width, height;
  gint arc_size = NOTIFICATION_ARC_SIZE;

  width = a->width; 
  height = a->height; 
	
  if (height > 60)
	  height = 60;
  
  x0 = y0 = 0;
  x1 = width - 1;
  y1 = height - 1;

  bitmap = gdk_pixmap_new (NULL, width, height, 1);
  
  zero_gc = gdk_gc_new (bitmap);
  one_gc = gdk_gc_new (bitmap);

  zero.pixel = 0;
  one.pixel = 1;

  gdk_gc_set_foreground (zero_gc, &zero);
  gdk_gc_set_foreground (one_gc, &one);

  gdk_draw_rectangle (bitmap, zero_gc, TRUE, 0, 0, width, height);

  /* North-west corner */
  gdk_draw_arc (bitmap, one_gc, TRUE, x0, y0, arc_size * 2, arc_size * 2, 90*64, 90*64);
  gdk_draw_arc (bitmap, one_gc, FALSE, x0, y0, arc_size * 2, arc_size * 2, 90*64, 90*64);
  /* North-east corner */
  gdk_draw_arc (bitmap, one_gc, TRUE, x1 - arc_size * 2, y0, arc_size * 2, arc_size * 2, 0*64, 90*64);
  gdk_draw_arc (bitmap, one_gc, FALSE, x1 - arc_size * 2, y0, arc_size * 2, arc_size * 2, 0*64, 90*64);
  /* South-west corner */
  gdk_draw_arc (bitmap, one_gc, TRUE, x0, y1 - arc_size * 2, arc_size * 2, arc_size * 2, 180*64, 90*64);
  gdk_draw_arc (bitmap, one_gc, FALSE, x0, y1 - arc_size * 2, arc_size * 2, arc_size * 2, 180*64, 90*64);
  /* South-east corner */
  gdk_draw_arc (bitmap, one_gc, TRUE, x1 - arc_size * 2, y1 - arc_size * 2, arc_size * 2, arc_size * 2, 270*64, 90*64);	
  gdk_draw_arc (bitmap, one_gc, FALSE, x1 - arc_size * 2, y1 - arc_size * 2, arc_size * 2, arc_size * 2, 270*64, 90*64);

  /* Middle */
  gdk_draw_rectangle (bitmap, one_gc, TRUE, x0 + arc_size, y0 + arc_size, width - arc_size * 2, height - arc_size * 2);

  /* Top */
  gdk_draw_rectangle (bitmap, one_gc, TRUE, x0 + arc_size, y0, width - arc_size * 2, arc_size);
  /* Bottom */
  gdk_draw_rectangle (bitmap, one_gc, TRUE, x0 + arc_size, y1 - arc_size, width - arc_size * 2, arc_size + 1);
  /* Left */
  gdk_draw_rectangle (bitmap, one_gc, TRUE, x0, y0 + arc_size, arc_size, height - arc_size * 2);
  /* Right */
  gdk_draw_rectangle (bitmap, one_gc, TRUE, x1 - arc_size, y0 + arc_size, arc_size + 1, height - arc_size * 2);

  g_object_unref (one_gc);
  g_object_unref (zero_gc);

  gtk_widget_shape_combine_mask (w, bitmap, 0, 0);

  g_object_unref (bitmap);
  return TRUE;
}

/* Draw and expose handler for the notification area window. */
static gboolean
handle_expose (GtkWidget *w, GdkEventExpose *ev, GtkWidget *child)
{	
  GdkGC *black_gc;
  GdkDrawable *dr;
  gint x0, y0, x1, y1;
  gint arc_size = NOTIFICATION_ARC_SIZE;

  black_gc = w->style->black_gc;
  dr = w->window;
  x0 = y0 = 0;
  x1 = w->allocation.width - 1;
  if (w->allocation.height > 60)		
	  y1 = 60 - 1;
  else
      y1 = w->allocation.height - 1;

  /* North-west corner */
  gdk_draw_arc (dr, black_gc, FALSE, x0, y0, arc_size * 2, arc_size * 2, 90*64, 90*64);
  /* North-east corner */
  gdk_draw_arc (dr, black_gc, FALSE, x1 - arc_size * 2, y0, arc_size * 2, arc_size * 2, 0*64, 90*64);
  /* South-west corner */
  gdk_draw_arc (dr, black_gc, FALSE, x0, y1 - arc_size * 2, arc_size * 2, arc_size * 2, 180*64, 90*64);
 /* South-east corner */
  gdk_draw_arc (dr, black_gc, FALSE, x1 - arc_size * 2, y1 - arc_size * 2, arc_size * 2, arc_size * 2, 270*64, 90*64);	

  /* Join them up */
  gdk_draw_line (dr, black_gc, x0 + arc_size, y0, x1 - arc_size, y0);
  gdk_draw_line (dr, black_gc, x0 + arc_size, y1, x1 - arc_size, y1);
  gdk_draw_line (dr, black_gc, x0, y0 + arc_size, x0, y1 - arc_size);
  gdk_draw_line (dr, black_gc, x1, y0 + arc_size, x1, y1 - arc_size);

  /* Draw the contents */
  gtk_container_propagate_expose (GTK_CONTAINER (w), child, ev);
  
  return TRUE;
}

/* Display a notification of the given type. */
void
homescreen_show_notification (gint type, const gchar *message)
{
  GtkWidget *popup;
  GtkWidget *icon, *label, *hbox;
  GdkColor  color;
  GdkGeometry geometry;
  PangoLayout *layout;
  GdkPixbuf *iconbuf;
  gint spacing = gpe_get_border ();
  gint xsize, lwidth, iwidth, ysize, lheight, iheight, xpos, ypos;
  gchar *msgstr;
  
  g_return_if_fail (message);

  msgstr = g_strdup_printf ("<b>%s</b>", message);
    
  popup = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  if (type == NOTE_MESSAGE_SENT) /* the popup is evanascent */
      g_timeout_add (2000, destroy_popup_evanascent, popup);
  else
      popup_list = g_list_append (popup_list, popup);
	
  gtk_window_set_type_hint (GTK_WINDOW (popup), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_modal (GTK_WINDOW (popup), TRUE);
  gtk_window_set_destroy_with_parent (GTK_WINDOW (popup), TRUE);
  gtk_window_set_decorated (GTK_WINDOW (popup), FALSE);
  gtk_window_set_default_size (GTK_WINDOW (popup), NOTIFICATION_WIDTH, 
                               NOTIFICATION_HEIGHT);
  gtk_widget_add_events (popup, GDK_BUTTON_PRESS_MASK | GDK_KEY_PRESS_MASK);
    
  gdk_color_parse ("#F8F884", &color);
  gtk_widget_modify_bg (popup, GTK_STATE_NORMAL, &color);
    
  switch (type)
    {
      case NOTE_ALARM:
        icon = gtk_image_new_from_pixbuf (gpe_find_icon ("msg_alarm"));
      break;
      case NOTE_CALL:
        icon = gtk_image_new_from_pixbuf (gpe_find_icon ("msg_call"));
      break;
	  case NOTE_MESSAGE:      
        icon = gtk_image_new_from_pixbuf (gpe_find_icon ("msg_message"));
      break;
	  case NOTE_MESSAGE_SENT:
		icon = gtk_image_new_from_pixbuf (gpe_find_icon ("msg_message"));
	  break;	  
	  default:
	    icon = gtk_image_new_from_pixbuf (gpe_find_icon ("msg_message"));
    }
  
  iconbuf = gtk_image_get_pixbuf (GTK_IMAGE (icon));
    
  label = gtk_label_new (NULL);
  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
  gtk_label_set_markup (GTK_LABEL (label), msgstr);
    
  hbox = gtk_hbox_new (FALSE, gpe_get_boxspacing ());
  gtk_container_set_border_width (GTK_CONTAINER (hbox), spacing);
    
  gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (popup), hbox);
  
  g_signal_connect (G_OBJECT (popup), "button-press-event", 
                            G_CALLBACK (popup_button_press_handler), GINT_TO_POINTER (type));
  g_signal_connect (G_OBJECT (popup), "key-press-event", 
                            G_CALLBACK (popup_key_press_handler), GINT_TO_POINTER (type));
  g_signal_connect (G_OBJECT (popup), "expose-event", G_CALLBACK (handle_expose), hbox);
  g_signal_connect (G_OBJECT (popup), "size-allocate", G_CALLBACK (handle_size_allocate), NULL);
    
  /* geometry setup */
    
  gtk_widget_realize (label);
 
  /* Calculate size used by the popup. */ 
  gtk_label_get_layout_offsets(GTK_LABEL(label), &xsize, &ysize);
  layout = gtk_label_get_layout(GTK_LABEL(label));
  pango_layout_get_pixel_size(layout, &lwidth, &lheight); 
  
  iwidth = gdk_pixbuf_get_width (iconbuf);
  iheight = gdk_pixbuf_get_height (iconbuf);
  xsize = xsize + lwidth + 6 + iwidth + gpe_get_boxspacing() + (spacing * 2);
  ysize = ysize + lheight + 6 + iheight;

  /* set final position and size */
  xpos = NOTIFICATION_XOFS;
  ypos = gdk_screen_height () / 2 + PANEL_SMALL;
  geometry.max_width = gdk_screen_width () - NOTIFICATION_XOFS * 2;
  geometry.min_width = gdk_screen_width () - NOTIFICATION_XOFS * 2;
  
  gtk_widget_set_uposition (GTK_WIDGET (popup), xpos, ypos);

  gtk_window_set_geometry_hints (GTK_WINDOW(popup), hbox, &geometry, 
                                 GDK_HINT_MAX_SIZE | GDK_HINT_MIN_SIZE);
    
  gtk_widget_show_all (hbox);
  gtk_window_present (GTK_WINDOW (popup));
  g_free (msgstr);
}

/* Theme change handler, loads theme default background image if no
 * custom wallpaper is set. */
void
homescreen_update_theme (const gchar *themename)
{
  gchar *bg_file = NULL;
  gchar *str = NULL;   
  gchar *current_theme = NULL;  
  gchar *resource_path = NULL;
  gchar *key = g_strconcat (THEME_PREFIX, themename, KEY_THEME_BG, NULL);
    
  setting_get_string (setting, key, &bg_file);
  g_free (key);
  if (bg_file)
    {    
      setting_get_string (setting, KEY_CURRENT_THEME, &current_theme);
      if (current_theme == NULL)
        current_theme = g_strdup ("default");
        
      setting_get_string (setting, KEY_RESOURCE_PATH, &resource_path);
      if (resource_path == NULL)
          resource_path = g_strdup (DEFAULT_RESOURCE_PATH);
      
      str = g_strconcat (resource_path, current_theme, "/", bg_file, NULL);  
      g_free (current_theme);
      g_free (resource_path);  
      g_free (bg_file);

      if (g_file_test (str, G_FILE_TEST_EXISTS) == FALSE)
        {
          g_free (str);
          str = g_strconcat (DEFAULT_PIXMAP_PATH, "/", bg_file, NULL);  
        }
 
      set_background_pixmap (str, homescreen.box);    
      g_free (str);
    }
  else
      set_background_pixmap (DEFAULT_THEME_BG, homescreen.box);    
}

/* Change the wallpaper to a custom file - handler used by settings. */
void
homescreen_update_bg_pixmap (const gchar *filename)
{
  set_background_pixmap (filename, homescreen.box);
}
  
/* Change the colour of the home screen background (used if no wallpaper is set. */
void
homescreen_update_bg_colour (GdkColor colour)
{
  set_background_colour (colour, homescreen.box);
}

/* Changes the status message of the home screen. */
void
homescreen_show_status (const gchar *statusmsg)
{
  gchar *text;
 
  if (homescreen.status_message != NULL)
      g_free (homescreen.status_message);
  homescreen.status_message = g_strdup (statusmsg);
  
  text = g_strdup_printf("<span foreground=\"%s\" size=\"16000\">%s</span>", 
                         homescreen.status_colour, statusmsg);
  gtk_label_set_markup (GTK_LABEL (homescreen.l_top), text);
  g_free (text);
}

void
homescreen_setup_qlaunch_btn (void)
{
  GtkWidget *mainwindow = applauncher_get_main_window();
    
  /* move the mouse pointer away from any button */
  XWarpPointer (GDK_WINDOW_XDISPLAY (mainwindow->window), 
                GDK_WINDOW_XID(mainwindow->window),
                None, 0, 0, 240, 320, 0, 60);
	
  if (GTK_IS_HBOX (homescreen.q_area))
    {
      GList *list = gtk_container_get_children (GTK_CONTAINER (homescreen.q_area));		
      if (g_list_length (list) > 0)
        {
          gtk_widget_grab_focus (GTK_WIDGET(list->data));
          list = g_list_next (list);
          while (list)
            {
              gtk_widget_set_state (GTK_WIDGET(list->data), GTK_STATE_NORMAL);
              list = g_list_next (list);
            }
        }			
    }
}

/* Load the configuration of the quick launch area from file. 
 * Returns the total amount of entries for the area. */
static gint
homescreen_load_quicklaunch_config (void)
{
  GKeyFile *configfile;
  GError *err = NULL;
  gchar **items, *aitem;
  gint num_items = 0;
	
  configfile = g_key_file_new();

  if (!g_key_file_load_from_file (configfile, FILE_CONFIG,
                                  G_KEY_FILE_KEEP_COMMENTS, &err))
	{
      fprintf(stderr, _("No quicklaunch items defined - disabling."));
	  g_error_free(err);
	  err = NULL;
      return FALSE;
	}
    
  items = g_key_file_get_groups (configfile, NULL);
  if (items == NULL)
      return FALSE;
  
  while (items[num_items])
    {
      gboolean error_occured = FALSE;
        
      homescreen.q_apps = g_realloc (homescreen.q_apps, sizeof (t_qlaunch_app) * (num_items + 1));
      memset (&homescreen.q_apps[num_items], 0, sizeof (t_qlaunch_app));
      aitem = items[num_items];
      
      homescreen.q_apps[num_items].name = 
          g_key_file_get_string(configfile, aitem, "Name", &err);
      if (err != NULL)
        {
            error_occured = TRUE;
            g_error_free (err);
            err = NULL;
        }
      homescreen.q_apps[num_items].icon = g_key_file_get_string(configfile, aitem, "Icon", &err);
      if (err != NULL)
        {
            error_occured = TRUE;
            g_error_free (err);
            err = NULL;
        }
      homescreen.q_apps[num_items].exec = g_key_file_get_string(configfile, aitem, "Exec", &err);
      if (err != NULL)
        {
            error_occured = TRUE;
            g_error_free (err);
            err = NULL;
        }
      if (error_occured == FALSE)
        num_items++;
    }
    
  g_strfreev(items);
  return (num_items);
}

/* Source function for timed update of the home screen labels including the 
 * big time label in the center of the screen. */
static gboolean
update_home_labels (gpointer userdata)
{
  char timebuf[40];
  time_t now = time(&now);
  struct tm *tm;
  gchar *text;
 
  tm = localtime (&now);

  /* update top status label */
  text = g_strdup_printf("<span foreground=\"%s\" size=\"22000\">%s</span>", 
                         homescreen.status_colour, homescreen.status_message);
  gtk_label_set_markup (GTK_LABEL (homescreen.l_top), text);
  g_free (text);
  
  /* update time label in the center */
  strftime (timebuf, 40, "%R", tm);
  text = g_strdup_printf("<span foreground=\"%s\" size=\"32000\">%s</span>", 
                         homescreen.label_colour, timebuf);
  gtk_label_set_markup (GTK_LABEL (homescreen.l_center), text);
  g_free (text);
  
  /* update date label */
  strftime (timebuf, 40, "%x", tm);
  text = g_strdup_printf("<span foreground=\"%s\" size=\"22000\">%s</span>", 
                         homescreen.label_colour, timebuf);
  gtk_label_set_markup (GTK_LABEL (homescreen.l_bottom), text);
  g_free (text);

  
  return TRUE; /* TRUE means to schedule this function again */
}

/* Start up a quicklaunch application. If the application is already running
 * its window is activated. */
static void
run_qlaunch_application (t_qlaunch_app *app)
{
  if (app->name == NULL) 
    app->name = g_strdup (app->exec);
  
  launch_application (app->name, app->exec, "", TRUE, TRUE, SERVICE_APPLAUNCHER);  	
}


/* Change the label colour of homscreen text widgets.
 * Takes symbolic names from rgb.txt or #rrggbb. */
void
homescreen_update_label_colour (const gchar *colour)
{
  if (homescreen.label_colour)
    g_free (homescreen.label_colour);
  homescreen.label_colour = g_strdup (colour);
  update_home_labels (NULL);
}
  
/* Change the status label colour of homscreen text widgets.
 * Takes symbolic names from rgb.txt or #rrggbb. */
void
homescreen_update_status_label_colour (const gchar *colour)
{
  if (homescreen.status_colour)
    g_free (homescreen.status_colour);
  homescreen.status_colour = g_strdup (colour);
  update_home_labels (NULL);
}

static void
focus_change (GtkContainer *container, GtkWidget *widget, gpointer user_data)
{
    
}

static gboolean 
icon_focus_in (GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
  gint index = GPOINTER_TO_INT(user_data);
    
  GdkPixbuf *pbuf = NULL;
	
  pbuf = load_icon_scaled(homescreen.q_apps[index].icon, 64);
  if (!pbuf)
    {
      pbuf = gpe_find_icon ("applauncher");
      pbuf = gdk_pixbuf_scale_simple(pbuf, 64, 64, GDK_INTERP_BILINEAR);
	}
       
  gtk_image_set_from_pixbuf(GTK_IMAGE(widget), pbuf);
					
  return FALSE;
}

static gboolean 
icon_focus_out (GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
  gint index = GPOINTER_TO_INT(user_data);
    
  GdkPixbuf *pbuf = NULL;
  gchar **strs = NULL;
  gchar *grey = NULL;
  strs = g_strsplit (homescreen.q_apps[index].icon, ".", 0);
  grey = g_strconcat (strs[0], "-grey.png", NULL);  	
  pbuf = load_icon_scaled(grey, 48);
  if (!pbuf)
    {
      pbuf = load_icon_scaled(homescreen.q_apps[index].icon, 48);
    }
  if (!pbuf)
    {
      pbuf = gpe_find_icon ("applauncher");
      pbuf = gdk_pixbuf_scale_simple(pbuf, 48, 48, GDK_INTERP_BILINEAR);
    }
        
  gtk_image_set_from_pixbuf(GTK_IMAGE(widget), pbuf);
  
  if (grey)					
    g_free (grey);
  if (strs);
    g_strfreev (strs);
    
  return FALSE;
}

static gboolean 
icon_key_press (GtkWidget *widget, GdkEventKey *event, gpointer	user_data)
{
  gint index = GPOINTER_TO_INT(user_data);
	
  if (event->keyval == GDK_Return)
    {
      t_qlaunch_app *app = &homescreen.q_apps[index];
        
      run_qlaunch_application(app);
		
      return TRUE;
    }
	
  return FALSE;
}

static gboolean 
icon_button_press (GtkWidget *widget, GdkEventButton *event, gpointer	user_data)
{
  gint index = GPOINTER_TO_INT(user_data);
	
  if (event->button == 1)
    {
      t_qlaunch_app *app = &homescreen.q_apps[index];
        
      run_qlaunch_application(app);
		
      return TRUE;
    }
	
  return FALSE;
}

static void 
settings_callback (int setting, int id, const char *key, type_t type, action_t action, void * user_data)
{  
  gchar *file = NULL;
#ifdef DEBUG    
  g_message ("homescreen callback for %s is called", key);
#endif
  setting_get_string (setting, key, &file);     
  set_background_pixmap (file, homescreen.box);
  if (file)  
    g_free (file);  
}

/* 
 * Create widgets for home area and initialize update. 
 * Returns the container of all the elements in homescreen.
 */
GtkWidget *
homescreen_create(GtkWidget *notebook)
{
  GtkWidget *vbox, *eventbox;
  GtkWidget *icon;
  GdkPixbuf *pbuf;
  GtkWidget *lspace1, *lspace2;
  gint num_ql_apps;
  gchar *current_theme = NULL; 
  gchar *bg_file = NULL;
  gboolean use_theme = FALSE;
  
  /* create and set up widgets */
  homescreen.label_colour = g_strdup ("white");
  homescreen.status_colour = g_strdup ("orange");
    
  homescreen.l_top = gtk_label_new (NULL);
  homescreen.l_center = gtk_label_new (NULL);
  homescreen.l_bottom = gtk_label_new (NULL);
  lspace1 = gtk_label_new (NULL);
  lspace2 = gtk_label_new (NULL);
    
  gtk_widget_set_size_request (homescreen.l_center, -1, gdk_screen_height() / 5);
  gtk_misc_set_alignment (GTK_MISC (homescreen.l_center), 0.5, 0.1);
  homescreen_show_status (DEFAULT_STATUSTEXT);
  vbox = gtk_vbox_new (FALSE, gpe_get_boxspacing());
    
  num_ql_apps = homescreen_load_quicklaunch_config ();
  if (num_ql_apps > 0)
    {
      gint i; 
      
      homescreen.q_area = gtk_hbox_new (TRUE, gpe_get_boxspacing());
      g_signal_connect (G_OBJECT (homescreen.q_area), "set-focus-child", 
                        G_CALLBACK (focus_change), NULL);
      for (i = 0; i < num_ql_apps; i++)
        {
          GtkWidget *icon = NULL;
          GdkPixbuf *pbuf = NULL;
          gchar **strs = NULL;
          gchar *grey = NULL;
          strs = g_strsplit (homescreen.q_apps[i].icon, ".", 0);
          grey = g_strconcat (strs[0], "-grey.png", NULL);  	
          pbuf = load_icon_scaled(grey, 48);
          if (!pbuf)
          {
  	        pbuf = load_icon_scaled(homescreen.q_apps[i].icon, 48);
          }
          if (grey)					
            g_free (grey);
          if (strs);
            g_strfreev (strs);
          if (!pbuf)
            {
              pbuf = gpe_find_icon ("applauncher");
              pbuf = gdk_pixbuf_scale_simple(pbuf, 48, 48, GDK_INTERP_BILINEAR);
            }
    
          icon = gtk_image_new_from_pixbuf(pbuf);
          gtk_widget_set_events(icon, GDK_ALL_EVENTS_MASK);
          GTK_WIDGET_SET_FLAGS(icon, GTK_CAN_FOCUS);
          gtk_widget_set_size_request (icon, 64, 64);
          gtk_widget_show(icon);
          gtk_widget_set_sensitive (icon, TRUE);
        
          g_signal_connect(G_OBJECT (icon), "key-press-event", 
                                             G_CALLBACK (icon_key_press), 
                                             GINT_TO_POINTER(i));
          g_signal_connect(G_OBJECT (icon), "button-press-event", 
                                             G_CALLBACK (icon_button_press), 
                                             GINT_TO_POINTER(i));
          g_signal_connect(G_OBJECT (icon), "focus-in-event", 
                                             G_CALLBACK (icon_focus_in), 
                                             GINT_TO_POINTER(i));
          g_signal_connect(G_OBJECT (icon), "focus-out-event", 
                                             G_CALLBACK (icon_focus_out), 
                                             GINT_TO_POINTER(i));
        
          gtk_box_pack_start(GTK_BOX(homescreen.q_area), icon, FALSE, TRUE, 0);
          if (i == 0)
            gtk_widget_grab_focus (icon);
        }
    }
  
  /* pack widgets into the screen */    
  if (num_ql_apps > 0)
      gtk_box_pack_start (GTK_BOX(vbox), homescreen.q_area, FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox), lspace1, FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox), homescreen.l_top, FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox), homescreen.l_center, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox), homescreen.l_bottom, FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox), lspace2, FALSE, TRUE, 0);

  pbuf = gdk_pixbuf_scale_simple (gpe_find_icon ("home"), TAB_ICON_SIZE, 
                                  TAB_ICON_SIZE, GDK_INTERP_BILINEAR);
  icon = gtk_image_new_from_pixbuf (pbuf);
  g_object_unref (pbuf);
    
  eventbox = gtk_event_box_new ();
  gtk_container_add (GTK_CONTAINER (eventbox), vbox);
    
  /* add update callback for displaying time */
  g_timeout_add (10000, update_home_labels, NULL);
  
  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), eventbox, icon);
  homescreen.box = eventbox;  

  /*
   * The public setting can be changed outside of theme, e.g. by a image viewer,
   * so we have to take care of this setting  
   */
  int notify_id;
  setting_notify_add (setting, KEY_BG_IMAGE, settings_callback, NULL, &notify_id);
    
  /* 
   * Set background image, try public setting first, the background image 
   * from current theme comes second. 
   */  
  setting_get_string (setting, KEY_CURRENT_THEME, &current_theme);
  if (current_theme == NULL)
    current_theme = g_strdup (DEFAULT_THEME);
    
  setting_get_string (setting, KEY_BG_IMAGE, &bg_file);
  if (bg_file == NULL)
    use_theme = TRUE;
  else if (access (bg_file, R_OK) != 0) /* read access to the file is denied or this file doesn't exist */
    use_theme = TRUE;  	
  
  if (use_theme)
    {
  	  gchar *key = g_strconcat (THEME_PREFIX, current_theme, KEY_THEME_BG, NULL);
  	  setting_get_string (setting, key, &bg_file);
  	  g_free (key);
    }
  g_free (current_theme);
  
  if (bg_file)
	{
      render_bg (bg_file, homescreen.box);    
      g_free (bg_file);
	}
  else
      render_bg (DEFAULT_THEME_BG, homescreen.box);
 
  /*
   * Grab the focus
   */
  gtk_widget_grab_focus (homescreen.box);
  update_home_labels (NULL);
  
  return eventbox;
}
